<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 10/04/19
 * Time: 11.09
 */

namespace FiloBlu\Rma\Handlers;

use Exception;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Registry;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Model\OrderRepository;
use Psr\Log\LoggerInterface;
use Throwable;

/**
 * Class DisablePersonalizedOrderRmaHanlder
 * @package FiloBlu\Rma\Handlers
 */
class DisablePersonalizedOrderRmaHanlder
{

    const RMA_CONFIG_PATH = 'filoblu_disablerma_section/rma_group/rma_option_id';
    /**
     * @var OrderRepository
     */
    protected $orderRepository;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var RequestInterface
     */
    protected $request;
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var Registry
     */
    private $registry;


    /**
     * DisablePersonalizedOrderRma constructor.
     * @param Context $context
     * @param OrderRepository $orderRepository
     * @param ScopeConfigInterface $scopeConfig
     * @param LoggerInterface $logger
     * @param Registry $registry
     */
    public function __construct(Context $context,
                                OrderRepository $orderRepository,
                                ScopeConfigInterface $scopeConfig,
                                LoggerInterface $logger,
                                Registry $registry)
    {
        $this->orderRepository = $orderRepository;
        $this->scopeConfig = $scopeConfig;
        $this->request = $context->getRequest();
        $this->logger = $logger;
        $this->registry = $registry;
    }


    /**
     * @param $result
     * @param null $order
     * @return bool
     */
    public function canCreateRmaForPersonalizedOrder($result, $currentOrder = null)
    {
        try {
            $config = $this->scopeConfig->getValue(self::RMA_CONFIG_PATH) ?? 0;
            $post = $this->request->getPostValue();
            if (!empty($post)) {
                //check in configuration if allow RMA request for personalized orders
                if (!$config) {
                    if (isset($post['items'][0]['order_item_id'])) {
                        foreach ($post['items'] as $postItem) {
                            $orderItemId = $postItem['order_item_id'];
                            try {
                                $order = $this->getOrder();
                                $allItems = $order->getAllItems();
                                $options = '';
                                //check if request has personalizations
                                foreach ($allItems as $item) {

                                    $options = $item->getProductOptions();
                                    //check if one of product is personalized and set Param to send to ShowCustomErrorMessageForRma.php
                                    if (($item->getItemId() == $orderItemId) && isset($options['options'])) {
                                        $result = false;
                                        $this->request->setParams(['isPersonalized' => true]);
                                    } else {
                                        $this->request->setParams(['isPersonalized' => false]);
                                    }
                                }

                            } catch (Exception $e) {
                                $this->request->setParams(['errorMessageFromPlugin' => $e]);
                            }
                        }
                    }
                }
            } else if ($this->request->getParam('order_id')) {
                $order = $this->getOrder();
                $myOrderitems = $order->getItems();
                $personalizedItems = [];
                if (!$config) {
                    foreach ($myOrderitems as $item) {
                        $options = $item->getProductOptions();
                        if (!empty($options['options'])) {
                            $personalizedItems [] = $item->getItemId();
                        }
                    }
                }
                $this->request->setParams(['personalizedItems' => $personalizedItems]);

            }
            return $result;
        } catch (Exception $e) {
            $this->logger->critical($e->getMessage());
        } catch (Throwable $t) {
            $this->logger->critical($t->getMessage());
        }

    }

    /**
     * @return OrderInterface|mixed|null
     * @throws InputException
     * @throws NoSuchEntityException
     */
    public function getOrder()
    {


        $orderId = $this->request->getParam('order_id');

        $order = $this->registry->registry("filoblu_rma_order_{$orderId}");

        if (!$order) {
            $order = $this->orderRepository->get($orderId);
            $this->registry->register("filoblu_rma_order_{$orderId}", $order);
        }

        return $order;
    }

}
